/*!@preserve
* ShareDir
*/

//import namespace
const fs          = require('fs')
const http        = require('http')
const path        = require('path')
const qs          = require('querystring')
const os          = require('os')
const cp          = require('child_process')

require('./sharedir.mount')

app.mod('sharedir', '../web')
app.pre('/sharedir', '.tmpl')

/*BLOG MENU*/
global.ONCEOS_DESKTOP_MENU.push({
    text   : 'SMB ' + LOCAL.SHARE_FOLDER
  , icon   : '/sharedir/img/sharedir.png'
  , href   : '/sharedir/index'
  , target : 'iframe'
})

global.ONCEOS_SUB_MENU_APP.nodes.push({
    text   : 'SMB ' + LOCAL.SHARE_FOLDER
  , icon   : '/sharedir/img/sharedir.png'
  , href   : '/sharedir/index'
})


/*
定义 sharedir schema

https://www.samba.org/samba/docs/using_samba/ch06.html
[share]
    path = /home/share
    browseable = yes
    comment = smb share test
    valid users = andy, dave, jay
    writable = no
    read only = yes 
    workgroup = METRAN
    guest ok = yes
*/
oncedb.schema('sdir', {
    "name"          : "id('sdir');minlen(1);maxlen(32)"
  , "path"          : "minlen(1);maxlen(256)"
  //, "writable"      : "maxlen(3)"  与 readonly 效果一样
  , "readonly"      : "maxlen(3)"
  , "guestok"       : "maxlen(3)"
  , "users"         : "maxlen(65535)"
  , "browseable"    : "maxlen(3)"
  , "workgroup"     : "maxlen(64)"
  , "cuser"         : "index('usdir', this.ctime || +new Date())"
  , "ctime"         : "int"
})


app.session('/sharedir', function(req, res) {
  var user = req.session.user

  if (!user || !user.username) {
    res.send(LOCAL.NO_PERMISSIONS)
    return
  }

  req.filter.next()
})


var selectPathItem = {
  class : 'form-group',
  child : [{
    tag       : 'label',
    class     : 'control-label col-sm-2',
    text      : LOCAL.FOLDER
  },{
    class     : 'col-sm-10',
    child     : [{
      class   : 'input-group',
      child   : [{
        name  : 'path',
        class : 'form-control',
        type  : 'text',
        placeholder: LOCAL.URL
      }, {
        tag     : 'span',
        class   : 'input-group-btn',
        attr    : { 'data-select-disk': '[name=path]' },
        html    : '<a class="btn btn-default"><i class="glyphicon glyphicon-ok-sign"></i> {0}</a>'.format(LOCAL.SELECT)
      }]
    }, {
      class   : 'text-right',
      html    : LOCAL.DO_NOT_USE_SYS_DISK
    }]
  }]
}

var sharedirFields = [{
    group: LOCAL.BASIC_INFORMATION
  , items: [{
      text    : LOCAL.NAME
    , name    : 'name'
  }, {
      html    : OnceDoc.json2html(selectPathItem)
  }, {
      text    : LOCAL.USER_LIST
    , name    : 'users'
    , tag     : 'input'
    , attr    : {
        'data-suggest-ajax'   : '/user/search'
      , 'data-display-field'  : 'id'
      , 'data-separator'      : ','
    }
  }, {
      text    : LOCAL.GUEST_OK
    , name    : 'guestok'
    , type    : 'checkbox'
  }, {
      text    : LOCAL.READONLY
    , name    : 'readonly'
    , type    : 'checkbox'
  }]
}, {
    group: LOCAL.MORE_INFORMATION
  , items: [{
      text    : 'workgroup'
    , name    : 'workgroup'
  }, {
      text    : 'browseable'
    , name    : 'browseable'
    , type    : 'checkbox'
    , attr    : { checked: 'checked' }
  }]
}]


var sharedirCols = [
    { title: LOCAL.NAME, field: 'name' }
  , { title: LOCAL.ACCESS_ADDRESS, field: 'address' }
  , { title: LOCAL.PATH, field: 'path' }
  , { title: LOCAL.READONLY, field: 'readonly' }
  , { title: LOCAL.GUEST_OK, field: 'guestok' }
  , { title: LOCAL.USER_LIST, field: 'users' }
  , { title: LOCAL.CREATOR, field: 'cuser' }
  , { title: LOCAL.CREATE_TIME, field: 'ctime', class: 'datetime' }
  , { title: LOCAL.MODIFY, field: 'editOpt' }
  , { title: LOCAL.DELETE, field: 'deleOpt' }
]

app.get('/sharedir/index', function(req, res) {
  res.render('sharedir.index.tmpl')
})

app.get('/sharedir/home', function(req, res) {
  var query       = req.query
  var pageNumber  = parseInt(query.pageNumber)  || 0
  var pageSize    = parseInt(query.pageSize)    || 100
  var user        = req.session.user            || {}

  var where = user.isAdmin ? {} : { cuser : user.username }

  oncedb.select('sdir', where, function(err, sharedirList) {
    if (err) {
      res.send(err.toString())
      return
    }

    var ip = OnceDoc.Address.ip()

    sharedirList.forEach(function(shareInfo, idx) {
      shareInfo.editOpt = '<a href="/sharedir/edit/{0}">{1}</a>'.format(shareInfo.name, LOCAL.MODIFY)
      shareInfo.deleOpt = '<a href="/sharedir/dele/{0}" data-confirm>{1}</a>'.format(shareInfo.name, LOCAL.DELETE)
      var address =  '\\\\' + ip + '\\' + shareInfo.name
      shareInfo.address = '<input readonly value="{0}" />'.format(address)
    })

    //remove site header&footer
    res.template({
        '/site.header.part' : ''
      , '/site.footer.part' : ''
    })

    res.render('/page.editList.tmpl', {
        cols        : sharedirCols
      , rows        : sharedirList
      , title       : LOCAL.SHARE_FOLDER
      , header      : LOCAL.SHARE_FOLDER
      , scripts     : [ '/sharedir/js/sharedir.home.js' ]
      , buttons     : [
            { html: app.render('/sharedir/sharedir.control.tmpl') }
          , { href: '/file/choose/disk', text: LOCAL.FILE_BROWSER, 'class': 'btn-default', target:'_blank' }
          , { href: '/sharedir/edit', text: LOCAL.CREATE_SMB_SHARED_FOLDER_SERVICE, 'class': 'btn-primary' }
        ]
      , pagination  : OnceDoc.getPagination({
            pageSize    : pageSize
          , pageNumber  : pageNumber
          , count       : sharedirList.count
          , urlFormat   : '/sharedir/home?pageNumber=${pageNumber}&pageSize=${pageSize}'
        })
    })
  }, { from: pageNumber * pageSize, to: (pageNumber + 1) * pageSize - 1, desc: true })

})


app.post('/sharedir/edit.post', function(req, res) {
  var shareDirInfo  = req.body || {}
  var user          = req.session.user || {}

  if (!user.username) {
    res.send({ error: LOCAL.SESSION_EXPIRED })
    return
  }

  if (!shareDirInfo.name || !shareDirInfo.path) {
    res.send({ error: LOCAL.INCOMPLETE_PARAMETER })
    return
  }

  if (!OnceDoc.isValidFileName(shareDirInfo.name)) {
    res.send({ error: LOCAL.NAME_INVALID })
    return
  }

  var render = function(err) {
    if (err) {
      res.send({ error: err.toString() })
      return
    }

    updateSMB(shareDirInfo, function(err) {
      if (err) {
        res.send({ error: err.toString() })
        return
      }

      res.send({ js: 'location.href = "/sharedir/home"' })
    })
  }

  var handle = function(err, states) {
    if (err) {
      res.send({ error: LOCAL.DOES_NOT_EXIST })
      return
    }

    if (!states || !states.isDirectory()) {
      res.send({ error: LOCAL.PLEASE_SELECT_FOLDER })
      return
    }

    oncedb.client.hgetall('sdir:' + shareDirInfo.name, function(err, oldSharedir) {
      if (err) {
        res.send({ error: err.toString() })
        return
      }

      oldSharedir = oldSharedir || {}

      console.log('oldSharedir', oldSharedir)
      console.log('shareDirInfo', shareDirInfo)

      if (shareDirInfo.IS_NEW) {
        if (oldSharedir.name) {
          res.send({ error: LOCAL.ALREADY_EXIST })
          return
        }

        shareDirInfo.cuser = user.username
        shareDirInfo.ctime = +new Date()

        oncedb.insert('sdir', shareDirInfo, render)
      } else {
        if (!oldSharedir.name) {
          res.send({ error: LOCAL.WRONG_PARAMETER })
          return
        }

        if (user.username != oldSharedir.cuser && !user.isAdmin) {
          res.send({ error: LOCAL.NO_PERMISSIONS })
          return
        }

        oncedb.update('sdir', shareDirInfo, render)
      }
    })
  }

  if (OnceDoc.isCurrentPath(shareDirInfo.path)) {
    //相对路径，先判断当前用户有无权限
    OnceDoc.hasPerm(req, path.join(rootDir, shareDirInfo.path), function(err, states) {
      if (err) {
        res.send({ error: err.toString() })
        return
      }

      handle(err, states)
    })
  } else {
    //非管理员使用了绝对路径
    if (!user.isAdmin) {
      res.send({ error: LOCAL.NO_PERMISSIONS })
      return
    }

    fs.stat(shareDirInfo.path, handle)
  }

})


app.get('/sharedir/edit/:name', function(req, res) {
  var name = req.params.name

  var render = function(err, shareList) {
    if (err) {
      res.send(err.toString())
      return
    }

    var shareInfo = shareList[0] || {}
    var isNew     = !shareInfo.path

    var nameFiled = sharedirFields[0].items[0]

    if (isNew) {
      delete nameFiled.readonly
    } else {
      nameFiled.readonly = true
    }

    res.render('/page.editField.tmpl', {
        header      : isNew ? LOCAL.ADD : LOCAL.EDIT
      , IS_NEW      : isNew
      // , EDIT_SCHEMA : 'user'
      , EDIT_FIELDS : sharedirFields
      , EDIT_DATA   : shareInfo
      , ACTION      : '/sharedir/edit.post'
      //, BACK_URL    : '/sharedir/home'  给通用action处理时才需要
      , buttons     : [
          { href: '/sharedir/home', text: LOCAL.LIST },
          { type: 'submit', 'class': 'btn-primary', text: LOCAL.SAVE }
        ]
    })
  }

  if (name) {
    oncedb.select('sdir', { name: name }, render)
  } else {
    render(null, [])
  }
})


app.get('/sharedir/dele/:name', function(req, res) {
  var user    = req.session.user || {}
  var name    = req.params.name

  if (!name || !OnceDoc.isValidFileName(name)) {
    res.send({ error: LOCAL.NAME_INVALID })
    return
  }

  oncedb.client.hmget('sdir:' + name, 'cuser', function(err, arrs) {
    if (err) {
      res.send({ error: err.toString() })
      return
    }

    //返回的为数组
    var sharename = arrs[0]

    if (user.username != sharename && !user.isAdmin) {
      res.send({ error: LOCAL.NO_PERMISSIONS })
      return
    }

    var shareDirInfo = { name: name }

    oncedb.remove('sdir', shareDirInfo, function(err) {
      if (err) {
        res.send({ error: err.toString() })
        return
      }

      //删除配置
      updateSMB(shareDirInfo, function() {
        res.send({ js: 'location.href = "/sharedir/home"' })
      })
    })
  })
})



var isWin32   = os.platform().toLowerCase() == 'win32'
var SMB_PATH  = isWin32 ? path.join(__dirname, 'smb.conf') : '/etc/samba/smb.conf'

var updateSysUser = function(shareDirInfo, cb) {
  if (isWin32) {
    cb && cb()
    return
  }

  var users = shareDirInfo.users
  if (!users || users.length < 1) {
    cb && cb()
    return
  }

  console.log('updateSysUser', users)

  OnceDoc.System.addGroup('onceair', function(err) {
    if (err) {
      console.log(err)
    }

    users.forEach(function(user) {
      oncedb.client.hget('user:' + user, 'password', function(err, encode) {
        if (err) {
          console.log(err)
          return
        }


        var password = OnceDoc.getDecryption(encode)


        //添加系统用户
        var addUserCMD = 'adduser --no-create-home --disabled-login --disabled-password --shell /usr/bin/git-shell --gecos "" --ingroup onceair ' + user
        cp.exec(addUserCMD, function(err, stdout, stderr) {
          if (err) {
            console.log(err)
          }

          //设置smb密码
          var smbpasswd = cp.spawn('smbpasswd', [ '-a', '-s', user ])
          smbpasswd.stderr.on('data', function(data) {
            console.log(data.toString())
          })

          //输入密码
          setTimeout(function() {
            //输入两次一样的密码
            smbpasswd.stdin.write(password + '\n' + password + '\n')
            smbpasswd.stdin.end()

            //'/etc/init.d/smbd restart'
          }, 400)
        })
      })
    })
  })
}

/*
start)
        /etc/init.d/nmbd start
        /etc/init.d/smbd start
        /etc/init.d/samba-ad-dc start
        ;;
stop)
        /etc/init.d/samba-ad-dc stop
        /etc/init.d/smbd stop
        /etc/init.d/nmbd stop
        ;;
reload)
        /etc/init.d/smbd reload
        ;;
restart|force-reload)
        /etc/init.d/nmbd "$1"
        /etc/init.d/smbd "$1"
        /etc/init.d/samba-ad-dc "$1"
*/
var smbTimer = null
var restartSMB = function() {
  if (isWin32) {
    return
  }

  if (smbTimer) {
    clearTimeout(smbTimer)
  }

  smbTimer = setTimeout(function() {
    smbTimer = null
    cp.exec('/etc/init.d/nmbd restart')
    cp.exec('/etc/init.d/smbd restart')
    cp.exec('/etc/init.d/samba-ad-dc restart')
  }, 10000)
}

var updateSMB = function(shareDirInfo, cb) {
  var onMod = function(err) {
    if (err) {
      cb && cb(err)
      return
    }

    fs.readFile(SMB_PATH, function(err, data) {
      if (err) {
        cb && cb(err)
        return
      }

      data = data.toString()

      var smbcfg  = data
      var smbname = '\n[' + shareDirInfo.name + ']'
      var start   = smbcfg.indexOf(smbname)
      var end
      var smbStr  = ''
      var smbnew

      //path 存在不添加，不存在则为删除
      if (shareDirInfo.path) {
        var smblines  = ['']
  
        smblines.push('[' + shareDirInfo.name + ']')
        smblines.push('    path = ' + dirPath)
        smblines.push('    writable = ' + (shareDirInfo.readonly ? 'no' : 'yes'))
        smblines.push('    browseable = ' + (shareDirInfo.browseable ? 'yes' : 'no'))

        if (shareDirInfo.workgroup) {
          smblines.push('    workgroup = ' + shareDirInfo.workgroup)
        }
  
        if (shareDirInfo.guestok) {
          smblines.push('    guestok = yes')
        } else if (shareDirInfo.users && shareDirInfo.users.length) {
          smblines.push('    valid users = ' + shareDirInfo.users.join(', '))
        }
  
        smblines.push('')
  
        smbStr = smblines.join('\n')
      }

      if (start > 0) {
        let stop = smbcfg.indexOf('\n\n', start + smbname.length + 1)

        if (stop < 0) {
          stop = smbcfg.length - 1
        }

        smbnew  = smbcfg.splice(start, stop - start + 1, smbStr)
      } else if (!smbStr) {
        //如果是删除，且配置文件无原来信息，直接返回
        cb && cb()
        return
      } else {
        smbnew  = smbcfg + smbStr
      }

      updateSysUser(shareDirInfo)

      fs.writeFile(SMB_PATH, smbnew, function(err) {
        if (err) {
          cb && cb(err)
          return
        }

        restartSMB()
        cb && cb()
      })
    })
  }

  if (shareDirInfo.path) {
    var dirPath = shareDirInfo.path
    if (OnceDoc.isCurrentPath(dirPath)) {
      dirPath = path.join(rootDir, dirPath)
      console.log('isCurrentPath', dirPath)
    }
  
    dirPath = path.resolve(dirPath)
    console.log('dirPath', dirPath)

    //八进制，更改目录读写权限
    fs.chmod(dirPath, 0777, onMod)
  } else {
    onMod()
  }
}


var getStatus = function(cb) {
  if (isWin32) {
    cb && cb(null, { running: false })
    return
  }

  cp.exec('ps aux | grep sbin/nmbd', function(err, stdout, stderr) {
    if (err) {
      cb && cb(err)
      return
    }

    var running   = stdout.toString().indexOf('/usr/sbin/nmbd') > 0

    cb && cb(null, { running: running })
  })
}

app.get('/sharedir/api/status', function(req, res) {
  getStatus(function(err, statusObj) {
    if (err) {
      res.send({ error: err.toString() })
      return
    }

    res.send(statusObj)
  })
})

app.get('/sharedir/api/service', function(req, res) {
  var query     = req.query || {}
  var running   = query.running == 'true'

  if (isWin32) {
    res.send({ error: LOCAL.FEATURE_DOES_NOT_SUPPORT_XX.format('Windows') })
    return
  }

  var status =  running ? 'start' : 'stop'

  cp.exec('/etc/init.d/nmbd ' + status, function(err) {
    if (err) {
      res.send({ error: err.toString() })
      return
    }

    cp.exec('/etc/init.d/smbd ' + status, function(err) {
      if (err) {
        res.send({ error: err.toString() })
        return
      }

      cp.exec('/etc/init.d/samba-ad-dc ' + status, function(err) {
        if (err) {
          res.send({ error: err.toString() })
          return
        }

        res.send({})
      })
    })
  })
})