/*!@preserve
* ShareDir
*/

//import namespace
var fs          = require('fs')
var http        = require('http')
var path        = require('path')
var qs          = require('querystring')
var os          = require('os')
var cp          = require('child_process')

oncedb.schema('smb', {
    "id"            : "id;sequence"
  , "smb"           : "minlen(1);maxlen(512)"
  , "dir"           : "maxlen(512)"
  , "inBrowser"     : ""
  , "isAnnon"       : ""
  , "user"          : ""
  , "pass"          : ""
  , "status"        : ""
  , "cuser"         : ""
  , "mtime"         : "int"
})

OnceDoc.on('ready', function() {
  //auto mount
  oncedb.select('smb', function(err, smbList) {
    if (err) {
      return
    }

    smbList.forEach(mountSmb)
  })
})

/*
https://www.cnblogs.com/faberbeta/p/samba003.html
mount -t cifs -o "rw,dir_mode=0644,file_mode=0644,username=username,password=yourpassword,vers=2.0" //192.168.1.100/yourshare_folder_name /usr/local/your_server_folder
*/
var mountSmb = function(mountInfo, cb) {
  var smbKey = 'smb' + mountInfo.id
  if (mountInfo.inBrowser) {
    mountInfo.dir = '/mnt/onceai/' + smbKey
  }

  mountInfo.smb = mountInfo.smb.replace(/\\/g, '/')

  fs.mkdir(mountInfo.dir, function(err) {
    //先制裁再挂载，否则重启进程，重新保存会显示挂载错误
    cp.exec('umount ' + mountInfo.dir, function(err) {
      var cmd = mountInfo.isAnnon
        ? 'mount -t cifs -o "rw,dir_mode=0644,file_mode=0644,vers=2.0" {smb} {dir}'
        : 'mount -t cifs -o "rw,dir_mode=0644,file_mode=0644,username={user},password={pass},vers=2.0" {smb} {dir}'

      cp.exec(cmd.format(mountInfo), function(err, stdout, stderr) {
        //隐去密码，防止泄露
        var errmsg = (stderr || '').toString()
        if (errmsg) {
          errmsg = errmsg.replace(/password=[\w]+,/, 'password=********,')
        }

        if (mountInfo.inBrowser) {
          ONCEDOC_DISK[smbKey] = {
              text      : LOCAL.NETWORK_DISK + ' ' + smbKey
            , device    : mountInfo.smb
            , mount     : mountInfo.dir
          }
        } else {
          delete ONCEDOC_DISK[smbKey]
        }

        oncedb.update('smb', { id: mountInfo.id, status: errmsg || 'OK', dir: mountInfo.dir }, function(err) {
          cb && cb(errmsg || err)
        })
      })
    })
  })
}

var mountFields = [{
    group: LOCAL.BASIC_INFORMATION
  , items: [{
      text      : LOCAL.ID
    , name      : 'id'
    , readonly  : true
  }, {
    html: OnceDoc.json2html({
      class : 'form-group',
      child : [{
        tag       : 'label',
        class     : 'control-label col-sm-2',
        text      : LOCAL.SMB_SHARED_FOLDER
      }, {
        class     : 'col-sm-10',
        child     : [{
          tag     : 'input',
          class   : 'form-control',
          name    : 'smb',
          attr    : { placeholder: LOCAL.EXAMPLE + ': //192.168.0.101/sharefolder' }
        }]
      }]
    })
  }, {
    html: OnceDoc.json2html({
      class : 'form-group',
      child : [{
        tag   : 'label',
        class : 'control-label col-sm-2'
      }, {
        class : 'col-sm-10',
        child : [{
          tag   : 'label',
          child : [{
            tag   : 'input',
            name  : 'inBrowser',
            type  : 'checkbox',
            attr  : { style: 'position: relative;top: 2px;' }
          }, {
            tag   : 'span',
            text  : LOCAL.SHOW_IN_FILE_BROWSER
          }]
        }]
      }]
    })
  }, {
    html: OnceDoc.json2html({
      class : 'form-group dir-wrap',
      child : [{
        tag       : 'label',
        class     : 'control-label col-sm-2',
        text      : LOCAL.LOCAL_FOLDER
      }, {
        class     : 'col-sm-10',
        child     : [{
          class   : 'input-group',
          child   : [{
            name  : 'dir',
            class : 'form-control',
            type  : 'text',
            placeholder: LOCAL.LOCAL_FOLDER
          }, {
            tag     : 'span',
            class   : 'input-group-btn',
            attr    : { 'data-select-disk': '[name=dir]' },
            html    : '<a class="btn btn-default"><i class="glyphicon glyphicon-ok-sign"></i> {0}</a>'.format(LOCAL.SELECT)
          }]
        }]
      }]
    })
  }, {
    html: OnceDoc.json2html({
      class : 'form-group',
      child : [{
        tag   : 'label',
        class : 'control-label col-sm-2'
      }, {
        class : 'col-sm-10',
        child : [{
          tag   : 'label',
          child : [{
            tag   : 'input',
            name  : 'isAnnon',
            type  : 'checkbox',
            attr  : { style: 'position: relative;top: 2px;' }
          }, {
            tag   : 'span',
            text  : LOCAL.ANON_ACCESS
          }]
        }]
      }]
    })
  }, {
    html: OnceDoc.json2html({
      class : 'form-group user-wrap',
      child : [{
        tag   : 'label',
        class : 'control-label col-sm-2',
        text  : LOCAL.USERNAME
      }, {
        class : 'col-sm-10',
        child : [{
          tag   : 'input',
          name  : 'user',
          class : 'form-control'
        }]
      }]
    })
  }, {
    html: OnceDoc.json2html({
      class : 'form-group user-wrap',
      child : [{
        tag   : 'label',
        class : 'control-label col-sm-2',
        text  : LOCAL.PASSWORD
      }, {
        class : 'col-sm-10',
        child : [{
          tag   : 'input',
          type  : 'password',
          name  : 'pass',
          class : 'form-control'
        }]
      }]
    })
  }]
}]

var mountCols = [
    { title: LOCAL.ID, field: 'id' }
  , { title: LOCAL.NETWORK_FOLDER, field: 'smb' }
  , { title: LOCAL.LOCAL_FOLDER, field: 'dir' }
  , { title: LOCAL.SHOW_IN_FILE_BROWSER, field: 'inBrowser' }
  , { title: LOCAL.ANON_ACCESS, field: 'isAnnon' }
  , { title: LOCAL.USERNAME, field: 'user' }
  , { title: LOCAL.PASSWORD, field: 'pass' }
  , { title: LOCAL.STATUS, field: 'status' }
  , { title: LOCAL.MODIFY_TIME, field: 'mtime', class: 'format-datetime' }
  , { title: LOCAL.MODIFY, field: 'editOpt' }
  , { title: LOCAL.DELETE, field: 'deleOpt' }
]

app.get('/sharedir/mount/home', function(req, res) {
  var query       = req.query
  var user        = req.session.user            || {}

  var render = function(err, mountList) {
    if (err) {
      res.send(err.toString())
      return
    }

    mountList.forEach(function(mountInfo, idx) {
      if (mountInfo.pass) {
        mountInfo.pass = '********'
      }

      mountInfo.editOpt = '<a href="/sharedir/mount/edit/{0}">{1}</a>'.format(mountInfo.id, LOCAL.MODIFY)
      mountInfo.deleOpt = '<a href="/sharedir/mount/dele/{0}" data-confirm>{1}</a>'.format(mountInfo.id, LOCAL.DELETE)
    })

    res.render('/page.editList.tmpl', {
        cols        : mountCols
      , rows        : mountList
      , title       : LOCAL.MOUNT_SMB_SHARED_FOLDER
      , header      : LOCAL.MOUNT_SMB_SHARED_FOLDER
      , buttons     : [
            { href: '/file/choose/disk', text: LOCAL.FILE_BROWSER, 'class': 'btn-default', target:'_blank' }
          , { href: '/sharedir/mount/edit', text: LOCAL.MOUNT_SMB_SHARED_FOLDER, 'class': 'btn-primary' }
        ]
    })
  }

  //只有管理员可以挂载
  if (user.isAdmin) {
    oncedb.select('smb', render, { desc: true })
  } else {
    render(null, [])
  }
})

app.post('/sharedir/mount/edit.post', function(req, res) {
  var shareDirInfo  = req.body || {}
  var user          = req.session.user || {}
  if (!user.isAdmin) {
    res.send(LOCAL.NO_PERMISSIONS)
    return
  }

  shareDirInfo.smb = (shareDirInfo.smb || '').trim()

  if (!shareDirInfo.smb || (!shareDirInfo.dir && !shareDirInfo.inBrowser)) {
    res.send({ error: LOCAL.INCOMPLETE_PARAMETER })
    return
  }

  var render = function(err) {
    if (err) {
      res.send({ error: err.toString() })
      return
    }

    mountSmb(shareDirInfo, function(err) {
      if (err) {
        console.log(err)
        //return res.send({ error: err.toString() })
      }

      res.send({ js: 'location.href = "/sharedir/mount/home"' })
    })
  }

  var handle = function() {
    shareDirInfo.mtime = +new Date()
    if (shareDirInfo.IS_NEW) {
      shareDirInfo.cuser = user.username
      oncedb.insert('smb', shareDirInfo, render)
    } else {
      oncedb.update('smb', shareDirInfo, render)
    }
  }

  if (!shareDirInfo.inBrowser) {
    fs.stat(shareDirInfo.dir, function(err, states) {
      if (err) {
        res.send({ error: err.toString() })
        return
      }

      if (!states.isDirectory()) {
        res.send({ error: LOCAL.PLEASE_SELECT_FOLDER })
        return
      }

      handle()
    })
  } else {
    handle()
  }
})

app.get('/sharedir/mount/edit/:id', function(req, res) {
  var id    = req.params.id
  var user  = req.session.user || {}
  if (!user.isAdmin) {
    res.send(LOCAL.NO_PERMISSIONS)
    return
  }

  var render = function(err, smbList) {
    if (err) {
      res.send(err.toString())
      return
    }

    var smbInfo = smbList[0] || {}
    var isNew   = isNaN(parseInt(smbInfo.id))

    res.render('/page.editField.tmpl', {
        header      : LOCAL.MOUNT_SMB_SHARED_FOLDER
      , IS_NEW      : isNew
      , EDIT_FIELDS : mountFields
      , EDIT_DATA   : smbInfo
      , ACTION      : '/sharedir/mount/edit.post'
      , scripts     : [ '/sharedir/js/sharedir.mount.edit.js' ]
      , buttons     : [
          { href: '/sharedir/mount/home', text: LOCAL.LIST },
          { type: 'submit', 'class': 'btn-primary', text: LOCAL.SAVE }
        ]
    })
  }

  if (id) {
    oncedb.select('smb', { id: id }, render)
  } else {
    render(null, [])
  }
})

app.get('/sharedir/mount/dele/:id', function(req, res) {
  var user  = req.session.user || {}
  var id    = req.params.id
  if (!user.isAdmin) {
    res.send(LOCAL.NO_PERMISSIONS)
    return
  }

  oncedb.select('smb', { id: id }, function(err, smbInfos) {
    if (err) {
      res.send({ error: err.toString() })
      return
    }

    var smbInfo = smbInfos[0]
    if (!smbInfo) {
      res.send({ error: LOCAL.WRONG_PARAMETER })
      return
    }

    var smbPath = smbInfo.dir
    if (smbInfo.inBrowser) {
      smbPath = '/mnt/onceai/smb' + id
    }

    if (smbPath) {
      cp.exec('umount ' + smbPath, function(err) {
        if (err) console.log(err)
      })
    }

    delete ONCEDOC_DISK['smb' + id]

    oncedb.remove('smb', { id: id }, function(err) {
      if (err) {
        res.send({ error: err.toString() })
        return
      }

      res.send({ js: 'location.href = "/sharedir/mount/home"' })
    })
  })
})